--- old/usr/src/uts/common/fs/nfs/nfs_export.c	2016-09-14 16:21:03.342391189 +0200
+++ new/usr/src/uts/common/fs/nfs/nfs_export.c	2016-09-14 16:21:03.346121570 +0200
@@ -1922,11 +1922,8 @@
 			break;
 		}
 
-		if (v4srv)
-			exi = checkexport4(&vp->v_vfsp->vfs_fsid, &fid, vp);
-		else
-			exi = checkexport(&vp->v_vfsp->vfs_fsid, &fid);
-
+		exi = checkexport(&vp->v_vfsp->vfs_fsid, &fid,
+		    v4srv ? vp : NULL);
 		if (exi != NULL) {
 			/*
 			 * Found the export info
@@ -2431,54 +2428,11 @@
 	return (vp);
 }
 
-/*
- * Find the export structure associated with the given filesystem.
- * If found, then increment the ref count (exi_count).
- */
 struct exportinfo *
-checkexport(fsid_t *fsid, fid_t *fid)
+checkexport_nohold(fsid_t *fsid, fid_t *fid, vnode_t *vp)
 {
 	struct exportinfo *exi;
 
-	rw_enter(&exported_lock, RW_READER);
-	for (exi = exptable[exptablehash(fsid, fid)];
-	    exi != NULL;
-	    exi = exi->fid_hash.next) {
-		if (exportmatch(exi, fsid, fid)) {
-			/*
-			 * If this is the place holder for the
-			 * public file handle, then return the
-			 * real export entry for the public file
-			 * handle.
-			 */
-			if (exi->exi_export.ex_flags & EX_PUBLIC) {
-				exi = exi_public;
-			}
-
-			exi_hold(exi);
-			rw_exit(&exported_lock);
-			return (exi);
-		}
-	}
-	rw_exit(&exported_lock);
-	return (NULL);
-}
-
-
-/*
- * "old school" version of checkexport() for NFS4.  NFS4
- * rfs4_compound holds exported_lock for duration of compound
- * processing.  This version doesn't manipulate exi_count
- * since NFS4 breaks fundamental assumptions in the exi_count
- * design.
- */
-struct exportinfo *
-checkexport4(fsid_t *fsid, fid_t *fid, vnode_t *vp)
-{
-	struct exportinfo *exi;
-
-	ASSERT(RW_LOCK_HELD(&exported_lock));
-
 	for (exi = exptable[exptablehash(fsid, fid)];
 	    exi != NULL;
 	    exi = exi->fid_hash.next) {
@@ -2502,15 +2456,33 @@
 			 * node for lofs and (pseudo) ufs may have
 			 * the same fsid and fid.
 			 */
-			if (vp == NULL || vp == exi->exi_vp)
+			if (vp == NULL || vp == exi->exi_vp) {
 				return (exi);
+			}
 		}
 	}
-
 	return (NULL);
 }
 
 /*
+ * Find the export structure associated with the given filesystem.
+ * If found, then increment the ref count (exi_count).
+ */
+struct exportinfo *
+checkexport(fsid_t *fsid, fid_t *fid, vnode_t *vp)
+{
+	struct exportinfo *exi;
+
+	rw_enter(&exported_lock, RW_READER);
+	exi = checkexport_nohold(fsid, fid, vp);
+	if (exi)
+		exi_hold(exi);
+	rw_exit(&exported_lock);
+
+	return (exi);
+}
+
+/*
  * Free an entire export list node
  */
 void
